# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import file


@cli.command("file-list", help="List all files.")
@click.pass_obj
def list(context):
    """List all files.

    >>> dcictl file-list
    """
    result = file.list(context)
    utils.format_output(result, context.format,
                        file.RESOURCE, file.TABLE_HEADERS)


@cli.command("file-show", help="Show a file.")
@click.option("--id", required=True, help="ID of the file to show")
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a file.

    >>> dcictl file-show [OPTIONS]

    :param string id: ID of the file to show [required]
    """
    result = file.get(context, id=id)
    utils.format_output(result, context['format'],
                        file.RESOURCE[:-1], file.TABLE_HEADERS)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import context
from dciclient.v1 import helper

from six.moves.urllib.parse import urlparse

import click
import requests


def _get_commit_from_tag(account_name, repo_name, tag_value):
    """Retrieve the commit of which a tag was made of"""

    api_url = "https://api.github.com/repos/%s/%s/tags" % (account_name,
                                                           repo_name)
    for tag in requests.get(api_url).json():
        if tag['name'] == tag_value:
            commit = tag['commit']['sha']
            break

    return commit


def _get_commit_from_base_url(url):
    """Retrieve the commit from the canonical url"""

    account_name, repo_name = urlparse(url).path[1:].split('/')
    api_url = "https://api.github.com/repos/%s/%s/branches" % (account_name,
                                                               repo_name)
    for branch in requests.get(api_url).json():
        if branch['name'] == 'master':
            commit = branch['commit']['sha']
            break

    return account_name, repo_name, commit


def get_components(topic_id, url):

    parsed_url = urlparse(url)
    if not parsed_url.scheme or not parsed_url.netloc == 'github.com':
        raise ValueError("must have a valid Github URL")

    # The code aims to support three distinc forms of URL
    #
    #   * https://github.com/redhat-cip/python-dciclient
    #   * https://github.com/redhat-cip/python-dciclient/commit/XXXXX
    #   * https://github.com/redhat-cip/python-dciclient/tree/0.1.1
    #
    # No matter what is the input form the output form should be
    #
    #  * https://api.github.com/repos/ORGA/REPO/commits/COMMIT
    #
    # The following methods aims to translate from the former to the
    # later
    if len(urlparse(url).path[1:].split('/')) == 2:
        account_name, repo_name, value = _get_commit_from_base_url(url)
    else:
        (account_name, repo_name, commit_type, value) = (
            parsed_url.path[1:].split('/')
        )
        if commit_type == 'tree':
            value = _get_commit_from_tag(account_name, repo_name, value)

    api_url = "https://api.github.com/repos/%s/%s/commits/%s" % (account_name,
                                                                 repo_name,
                                                                 value)
    commit_data = requests.get(api_url).json()

    github_component = {
        "type": component.GIT_COMMIT,
        "name": '%s - %s' % (repo_name, value),
        "canonical_project_name": repo_name,
        "sha": value,
        "title": commit_data['commit']['message'].split('\n')[0],
        "message": commit_data['commit']['message'],
        "url": url,
        "git": 'https://github.com/%s/%s.git' % (account_name, repo_name),
        "topic_id": topic_id
    }

    return [github_component]


@click.command()
@click.option('--dci-login', envvar='DCI_LOGIN', required=True,
              help="DCI username account.")
@click.option('--dci-password', envvar='DCI_PASSWORD', required=True,
              help="DCI password account.")
@click.option('--dci-cs-url', envvar='DCI_CS_URL', required=True,
              help="DCI CS url.")
@click.option('--dci-topic-id', envvar='DCI_TOPIC_ID', required=True,
              help="DCI topic id.")
@click.option('--github-url', envvar='DCI_FEEDER_GITHUB_URL', required=True,
              help="Github URL to feed to the control-server.")
def main(dci_login, dci_password, dci_cs_url, dci_topic_id, github_url):

    dci_context = context.build_dci_context(dci_cs_url, dci_login,
                                            dci_password)

    components = get_components(dci_topic_id, github_url)
    test_id = helper.get_test_id(dci_context, components[0]['name'],
                                 dci_topic_id)

    helper.create_jobdefinition_and_add_component(dci_context, components,
                                                  test_id, dci_topic_id)


if __name__ == '__main__':
    main()



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import jobdefinition


@cli.command("jobdefinition-list", help="List all jobdefinitions.")
@click.option("--topic_id", required=True)
@click.pass_obj
def list(context, topic_id):
    """list(context, id)

    List all jobdefinitions.

    >>> dcictl jobdefinition-list [OPTIONS]

    :param string topic_id: Topic ID [required]
    """
    result = jobdefinition.list(context, topic_id)
    utils.format_output(result, context.format,
                        jobdefinition.RESOURCE,
                        jobdefinition.TABLE_HEADERS)


@cli.command("jobdefinition-create", help="Create a jobdefinition.")
@click.option("--name", required=True)
@click.option("--test_id", required=True)
@click.option("--topic_id", required=True)
@click.option("--priority")
@click.pass_obj
def create(context, name, test_id, priority, topic_id):
    """create(context, name, test_id, priority, topic_id)

    Create a jobdefinition.

    >>> dcictl jobdefinition-create [OPTIONS]

    :param string name: Name of the jobdefinition [required]
    :param string test_id: ID of the test for this jobdefinition [required]
    :param string topic_id: ID of the topic for this jobdefinition [required]
    :param integer priority: Priority for this jobdefinition
    """
    result = jobdefinition.create(context, name=name, test_id=test_id,
                                  priority=priority, topic_id=topic_id)
    utils.format_output(result, context.format,
                        jobdefinition.RESOURCE[:-1])


@cli.command("jobdefinition-delete", help="Delete a jobdefinition.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a jobdefinition.

    >>> dcictl jobdefinition-delete [OPTIONS]

    :param string id: ID of the jobdefinition to delete [required]
    :param string etag: Entity tag of the jobdefinition resource [required]
    """
    result = jobdefinition.delete(context, id=id, etag=etag)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job Definition deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("jobdefinition-show", help="Show a jobdefinition.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a jobdefinition.

    :param string id: ID of the jobdefinition to show [required]
    """
    result = jobdefinition.get(context, id=id)
    utils.format_output(result, context.format,
                        jobdefinition.RESOURCE[:-1],
                        jobdefinition.TABLE_HEADERS)


@cli.command("jobdefinition-attach-component",
             help="Attach a component to a jobdefinition.")
@click.option("--id", required=True)
@click.option("--component_id", required=True)
@click.pass_obj
def attach_component(context, id, component_id):
    """attach_component(context, id, component_id)

    Attach a component to a jobdefinition.

    >>> dcictl jobdefinition-attach-component [OPTIONS]

    :param string id: ID of the jobdefinition to attach the component to
                      [required]
    :param string component_id: ID of the component to attach [required]
    """
    result = jobdefinition.add_component(context, id=id,
                                         component_id=component_id)
    utils.format_output(result, context.format)


@cli.command("jobdefinition-unattach-component",
             help="Unattach a component to a jobdefinition.")
@click.option("--id", required=True)
@click.option("--component_id", required=True)
@click.pass_obj
def unattach_component(context, id, component_id):
    """unattach_component(context, id, component_id)

    Unattach a component from a jobdefinition.

    >>> dcictl jobdefinition-unattach-component [OPTIONS]

    :param string id: ID of the jobdefinition to unattach the component from
                      [required]
    :param string component_id: ID of the component to unattach [required]
    """
    result = jobdefinition.remove_component(context, id=id,
                                            component_id=component_id)
    if result.status_code == 204:
        unattach_msg = 'Component unattached from Jobdefinition'
        utils.print_json({'id': id,
                          'message': unattach_msg})
    else:
        utils.format_output(result, context.format)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import context
from dciclient.v1 import helper

from six.moves.urllib.parse import urlparse

import click
import re
import requests


def get_repo_information(repo_file):
    repo_file_raw_content = requests.get(repo_file).text
    base_url = re.search('baseurl=((.*)/os)', repo_file_raw_content).group(1)
    base_url = base_url.replace("$basearch", "x86_64")
    version = urlparse(base_url).path.split('/')[4]
    repo_name = urlparse(base_url).path.split('/')[5]

    return base_url, version, repo_name


def get_puddle_component(repo_file, topic_id):
    (base_url, version, repo_name) = get_repo_information(repo_file)

    puddle_component = {
        'type': component.PUDDLE,
        'canonical_project_name': repo_name,
        'name': '%s %s' % (repo_name, version),
        'url': base_url,
        'data': {
            'path': urlparse(base_url).path,
            'version': version,
            'repo_name': repo_name
        },
        'topic_id': topic_id,
    }
    return puddle_component


@click.command()
@click.option('--dci-login', envvar='DCI_LOGIN', required=True,
              help="DCI username account.")
@click.option('--dci-password', envvar='DCI_PASSWORD', required=True,
              help="DCI password account.")
@click.option('--dci-cs-url', envvar='DCI_CS_URL', required=True,
              help="DCI CS url.")
@click.option('--dci-topic-id', envvar='DCI_TOPIC_ID', required=True,
              help="DCI topic id.")
def main(dci_login, dci_password, dci_cs_url, dci_topic_id):
    dci_context = context.build_dci_context(dci_cs_url, dci_login,
                                            dci_password)

    # Create Khaleesi-tempest test
    test_id = helper.get_test_id(dci_context, 'tempest', dci_topic_id)

    components = [
        # TODO(Gonéri): We should also return the images.
        get_puddle_component(
            'http://download.eng.bos.redhat.com/rel-eng/OpenStack/' +
            '8.0-RHEL-7-director/latest/RH7-RHOS-8.0-director.repo',
            dci_topic_id),
        get_puddle_component(
            'http://download.eng.bos.redhat.com/rel-eng/OpenStack/' +
            '8.0-RHEL-7/latest/RH7-RHOS-8.0.repo',
            dci_topic_id)]

    helper.create_jobdefinition_and_add_component(dci_context, components,
                                                  test_id, dci_topic_id)

if __name__ == '__main__':
    main()



# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1 import utils

import json


def create(context, resource, **kwargs):
    """Create a resource"""
    data = utils.sanitize_kwargs(**kwargs)
    uri = '%s/%s' % (context.dci_cs_api, resource)
    r = context.session.post(uri, data=json.dumps(data))
    return r


def list(context, resource, **kwargs):
    """List all resources"""
    data = utils.sanitize_kwargs(**kwargs)
    topic_id = data.pop('topic_id', None)

    if topic_id is not None:
        uri = '%s/topics/%s/%s' % (context.dci_cs_api, topic_id, resource)
    else:
        uri = '%s/%s' % (context.dci_cs_api, resource)

    r = context.session.get(uri, params=data)
    return r


def get(context, resource, **kwargs):
    """List a specific resource"""
    uri = '%s/%s/%s' % (context.dci_cs_api, resource, kwargs.pop('id'))
    r = context.session.get(uri, params=kwargs)
    return r


def update(context, resource, **kwargs):
    """Update a specific resource"""
    etag = kwargs.pop('etag')
    id = kwargs.pop('id')
    data = utils.sanitize_kwargs(**kwargs)
    uri = '%s/%s/%s' % (context.dci_cs_api, resource, id)
    r = context.session.put(uri, headers={'If-match': etag}, json=data)
    return r


def delete(context, resource, **kwargs):
    """Delete a specific resource"""

    etag = kwargs.pop('etag', None)
    uri = '%s/%s/%s' % (context.dci_cs_api, resource, kwargs.pop('id'))
    r = context.session.delete(uri, headers={'If-match': etag})
    return r



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import team


@cli.command("team-list", help="List all teams.")
@click.pass_obj
def list(context):
    """List all teams.

    >>> dcictl team list
    """
    result = team.list(context)
    utils.format_output(result, context.format,
                        team.RESOURCE, team.TABLE_HEADERS)


@cli.command("team-create", help="Create a team.")
@click.option("--name", required=True)
@click.pass_obj
def create(context, name):
    """create(context, name)

    Create a team.

    >>> dcictl team-create [OPTIONS]

    :param string name: Name of the team [required]
    """
    result = team.create(context, name=name)
    utils.format_output(result, context.format, team.RESOURCE[:-1])


@cli.command("team-update", help="Update a team.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.option("--name", required=True)
@click.pass_obj
def update(context, id, etag, name):
    """update(context, id, etag, name)

    Update a team.

    >>> dcictl team-update [OPTIONS]

    :param string id: ID of the team to update [required]
    :param string etag: Entity tag of the resource [required]
    :param string name: Name of the team [required]
    """
    result = team.update(context, id=id, etag=etag, name=name)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Team updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("team-delete", help="Delete a team.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a team

    >>> dcictl team-delete [OPTIONS]

    :param string id: ID of the team to delete [required]
    :param string etag: Entity tag of resource [required]
    """
    result = team.delete(context, id=id, etag=etag)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Team deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("team-show", help="Show a team.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a team.

    >>> dcictl team-show [OPTIONS]

    :param string id: ID of the team [required]
    """
    result = team.get(context, id=id)
    utils.format_output(result, context.format,
                        team.RESOURCE[:-1], team.TABLE_HEADERS)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import test

import json


@cli.command("test-list", help="List all tests.")
@click.option("--topic_id", required=True)
@click.pass_obj
def list(context, topic_id):
    """List all tests.

    >>> dcictl test list
    """
    result = test.list(context, topic_id)
    utils.format_output(result, context.format,
                        test.RESOURCE, test.TABLE_HEADERS)


@cli.command("test-create", help="Create a test.")
@click.option("--name", required=True)
@click.option("--topic_id", required=True)
@click.option("--data", default='{}')
@click.pass_obj
def create(context, name, data, topic_id):
    """Create a test.

    >>> dcictl test-create [OPTIONS]

    :param string name: Name of the test [required]
    :param string topic_id: ID of the topic to associate with [required]
    :param json data: JSON formatted data block for the test
    """
    data = json.loads(data)
    result = test.create(context, name=name, data=data, topic_id=topic_id)
    utils.format_output(result, context.format, test.RESOURCE[:-1])


@cli.command("test-delete", help="Delete a test.")
@click.option("--id", required=True)
@click.pass_obj
def delete(context, id):
    """Delete a test.

    >>> dcictl test-delete [OPTIONS]

    :param string id: ID of the test to delete [required]
    """
    result = test.delete(context, id=id)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Test deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("test-show", help="Show a test.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """Show a test.

    >>> dcictl test-show [OPTIONS]

    :param string id: ID of the test to return [required]
    """
    result = test.get(context, id=id)
    utils.format_output(result, context.format,
                        test.RESOURCE[:-1], test.TABLE_HEADERS)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import io
import logging
import threading

from dciclient.v1.api import file as dci_file
from dciclient.v1.api import jobstate as dci_jobstate


class DciHandler(logging.Handler):
    def __init__(self, dci_context, info_as_jobstate=False):
        logging.Handler.__init__(self)
        self._info_as_jobstate = info_as_jobstate
        self._dci_context = dci_context
        self._idx_file = 0
        self._current_log = io.StringIO()
        self._threshold_log = 512 * 1024  # 512K
        self._interval = 30  # 30 seconds
        self._start_timer()

    def _start_timer(self):
        timer_handle = functools.partial(self.emit, record=None)
        self._timer = threading.Timer(self._interval, timer_handle)
        self._timer.setDaemon(True)
        self._timer.start()

    def _send_log_file(self):
        if not self._dci_context.last_jobstate_id:
            return
        jobstate_id = self._dci_context.last_jobstate_id
        dci_file.create(self._dci_context, '%s.log' % self._idx_file,
                        self._current_log.getvalue(), 'text/plain',
                        jobstate_id)
        self._current_log.truncate(0)
        self._current_log.seek(0)
        self._idx_file += 1

    def emit(self, record):
        # if record is None then emit() is actually run by the timer
        if record is None:
            self.acquire()
            try:
                if len(self._current_log.getvalue()) > 0:
                    self._send_log_file()
            finally:
                self.release()
            self._start_timer()
            return

        msg = u"%s\n" % self.format(record)

        if record.levelno == logging.INFO:
            if self._info_as_jobstate and self._dci_context.last_job_id:
                dci_jobstate.create(
                    self._dci_context,
                    'running',
                    msg.rstrip('\n'), self._dci_context.last_job_id)
                return

        self._current_log.write(msg)
        #  if its an error then send the log
        if record.levelno == logging.ERROR:
            self._send_log_file()
        #  if we reach the current log threshold
        elif len(self._current_log.getvalue()) > self._threshold_log:
            self._send_log_file()



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import dci
import dci.app
import dci.dci_config

from dciclient.v1.api import component
from dciclient.v1.api import context
from dciclient.v1.api import job
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import remoteci
from dciclient.v1.api import team
from dciclient.v1.api import test
from dciclient.v1.api import topic

import pytest
import sqlalchemy
import sqlalchemy_utils.functions

from dciclient import v1 as dci_client

import click.testing
import dciclient.shell as shell
from dciclient.v1.tests.shell_commands import utils
import functools


@pytest.fixture(scope='session')
def engine(request):
    conf = dci.dci_config.generate_conf()
    db_uri = conf['SQLALCHEMY_DATABASE_URI']

    engine = sqlalchemy.create_engine(db_uri)

    def del_db():
        if sqlalchemy_utils.functions.database_exists(db_uri):
            sqlalchemy_utils.functions.drop_database(db_uri)

    del_db()
    request.addfinalizer(del_db)
    sqlalchemy_utils.functions.create_database(db_uri)

    dci.db.models.metadata.create_all(engine)
    return engine


@pytest.fixture
def db_clean(request, engine):
    def fin():
        for table in reversed(dci.db.models.metadata.sorted_tables):
            engine.execute(table.delete())
    request.addfinalizer(fin)


@pytest.fixture
def db_provisioning(db_clean, engine):
    with engine.begin() as conn:
        utils.provision(conn)


@pytest.fixture
def server(db_provisioning, engine):
    app = dci.app.create_app(dci.dci_config.generate_conf())
    app.testing = True
    app.engine = engine
    return app


@pytest.fixture
def client(server, db_provisioning):
    client = dci_client.DCIClient(
        end_point='http://dci_server.com/api',
        login='admin', password='admin'
    )
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    client.s.mount('http://dci_server.com', flask_adapter)
    return client


@pytest.fixture
def dci_context(server, db_provisioning):
    test_context = context.DciContext('http://dci_server.com',
                                      'admin', 'admin')
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    test_context.session.mount('http://dci_server.com', flask_adapter)
    return test_context


@pytest.fixture
def runner(dci_context):
    context.build_dci_context = lambda **kwargs: dci_context
    runner = click.testing.CliRunner(env={'DCI_LOGIN': '', 'DCI_PASSWORD': '',
                                          'DCI_CLI_OUTPUT_FORMAT': 'json'})
    runner.invoke = functools.partial(runner.invoke, shell.main)
    return runner


@pytest.fixture
def team_id(dci_context):
    return team.create(dci_context, name='tname').json()['team']['id']


@pytest.fixture
def topic_id(dci_context):
    return topic.create(dci_context, name='foo_topic').json()['topic']['id']


@pytest.fixture
def test_id(dci_context, topic_id):
    return test.create(dci_context, name='test_name',
                       topic_id=topic_id).json()['test']['id']


@pytest.fixture
def components(dci_context, topic_id):
    component1 = {'name': 'component1',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 1',
                  'topic_id': topic_id}

    component2 = {'name': 'component2',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 2',
                  'topic_id': topic_id}

    return [component1, component2]


@pytest.fixture
def remoteci_id(dci_context):
    team_id = team.create(dci_context, name='tname').json()['team']['id']
    rci = remoteci.create(dci_context, name='remoteci', team_id=team_id).json()
    return rci['remoteci']['id']


@pytest.fixture
def component_id(dci_context, topic_id):
    my_component = component.create(
        dci_context, name='component1', type='git_review',
        data={'component': 'component'}, topic_id=topic_id).json()
    return my_component['component']['id']


@pytest.fixture
def job_id(dci_context):
    my_topic = topic.create(dci_context, name='topic_name').json()['topic']
    my_team = team.create(dci_context, name='tname').json()['team']
    team_id = team.list(dci_context).json()['teams'][0]['id']
    topic.attach_team(dci_context, my_topic['id'], team_id)
    my_remoteci = remoteci.create(dci_context,
                                  name='tname', team_id=my_team['id'],
                                  data={'remoteci': 'remoteci'}).json()
    my_remoteci_id = my_remoteci['remoteci']['id']
    my_test = test.create(
        dci_context,
        topic_id=my_topic['id'],
        name='tname',
        data={'test': 'test'}).json()
    my_test_id = my_test['test']['id']
    my_jobdefinition = jobdefinition.create(
        dci_context,
        name='tname',
        test_id=my_test_id,
        topic_id=my_topic['id']).json()
    my_component = component.create(
        dci_context, name='hihi', type='git_review',
        data={'component': 'component'}, topic_id=my_topic['id']).json()
    my_component_id = my_component['component']['id']
    jobdefinition.add_component(dci_context,
                                my_jobdefinition['jobdefinition']['id'],
                                my_component_id)
    my_job = job.schedule(dci_context, my_remoteci_id, my_topic['id']).json()
    return my_job['job']['id']



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import team
from dciclient.v1.api import topic

import click


@cli.command("topic-list", help="List all topics.")
@click.pass_obj
def list(context):
    """List all topics.

    >>> dcictl topic-list
    """
    topics = topic.list(context)
    utils.format_output(topics, context.format,
                        topic.RESOURCE, topic.TABLE_HEADERS)


@cli.command("topic-create", help="Create a topic.")
@click.option("--name", required=True)
@click.pass_obj
def create(context, name):
    """create(context, name)

    Create a topic.

    >>> dcictl topic-create [OPTIONS]

    :param string name: Name of the topic [required]
    """
    result = topic.create(context, name=name)
    utils.format_output(result, context.format, topic.RESOURCE[:-1])


@cli.command("topic-delete", help="Delete a topic.")
@click.option("--id", required=True)
@click.pass_obj
def delete(context, id):
    """delete(context, id)

    Delete a topic.

    >>> dcictl topic-delete [OPTIONS]

    :param string id: ID of the topic to delete [required]
    """
    result = topic.delete(context, id=id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Topic deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("topic-show", help="Show a topic.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a topic.

    >>> dcictl topic-show [OPTIONS]

    :param string id: ID of the topic to return [required]
    """
    result = topic.get(context, id=id)
    utils.format_output(result, context.format, topic.RESOURCE[:-1],
                        topic.TABLE_HEADERS)


@cli.command("topic-attach-team", help="Attach a team to a topic.")
@click.option("--id", required=True)
@click.option("--team_id", required=True)
@click.pass_obj
def attach_team(context, id, team_id):
    """attach_team(context, id, team_id)

    Attach a team to a topic.

    >>> dcictl topic-attach-team [OPTIONS]

    :param string id: ID of the topic to attach to [required]
    :param string team_id: ID of the team to attach to this topic [required]
    """
    result = topic.attach_team(context, id=id, team_id=team_id)
    utils.format_output(result, context.format)


@cli.command("topic-unattach-team", help="Unattach a team from a topic.")
@click.option("--id", required=True)
@click.option("--team_id", required=True)
@click.pass_obj
def unattach_team(context, id, team_id):
    """unattach_team(context, id, team_id)

    Unattach a team from a topic.

    >>> dcictl topic-unattach-team [OPTIONS]

    :param string id: ID of the topic to unattach from [required]
    :param string team_id: ID of team to unattach from this topic [required]
    """
    result = topic.unattach_team(context, id=id, team_id=team_id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Teams has been unattached.'})
    else:
        utils.format_output(result, context.format)


@cli.command("topic-list-team", help="List teams attached to a topic.")
@click.option("--id", required=True)
@click.pass_obj
def list_attached_team(context, id):
    """list_attached_team(context, id)

    List teams attached to a topic.

    >>> dcictl topic-list-team

    :param string id: ID of the topic to list teams for [required]
    """
    result = topic.list_attached_team(context, id=id)
    utils.format_output(result, context.format,
                        team.RESOURCE, team.TABLE_HEADERS)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import remoteci

import json


@cli.command("remoteci-list", help="List all remotecis.")
@click.pass_obj
def list(context):
    """List all Remote CIs

    >>> dcictl remoteci-list
    """
    result = remoteci.list(context)
    utils.format_output(result, context.format,
                        remoteci.RESOURCE, remoteci.TABLE_HEADERS)


@cli.command("remoteci-create", help="Create a remoteci.")
@click.option("--name", required=True)
@click.option("--team_id", required=True)
@click.option("--data", default='{}')
@click.option("--active/--no-active", default=True)
@click.pass_obj
def create(context, name, team_id, data, active):
    """create(context, name, team_id, data, active)

    Create a Remote CI

    >>> dcictl remoteci-create [OPTIONS]

    :param string name: Name of the Remote CI [required]
    :param string team_id: ID of the team to associate this remote CI with
        [required]
    :param string data: JSON data to pass during remote CI creation
    :param boolean active: Mark remote CI active
    :param boolean no-active: Mark remote CI inactive
    """
    data = json.loads(data)
    result = remoteci.create(context, name=name, team_id=team_id, data=data,
                             active=active)
    utils.format_output(result, context.format, remoteci.RESOURCE[:-1])


@cli.command("remoteci-update", help="Update a remoteci.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.option("--name")
@click.option("--team_id")
@click.option("--data")
@click.option("--active/--no-active")
@click.pass_obj
def update(context, id, etag, name, team_id, data, active):
    """update(context, id, etag, name, team_id, data, active)

    Update a Remote CI.

    >>> dcictl remoteci-update [OPTIONS]

    :param string id: ID of the remote CI [required]
    :param string etag: Entity tag of the remote CI resource [required]
    :param string name: Name of the Remote CI
    :param string team_id: ID of the team to associate this remote CI with
    :param string data: JSON data to pass during remote CI update
    :param boolean active: Mark remote CI active
    :param boolean no-active: Mark remote CI inactive
    """
    result = remoteci.update(context, id=id, etag=etag, name=name,
                             team_id=team_id, data=data, active=active)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Remote CI updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("remoteci-delete", help="Delete a remoteci.")
@click.option("--id", required=True)
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a Remote CI.

    >>> dcictl remoteci-delete [OPTIONS]

    :param string id: ID of the remote CI to delete [required]
    :param string etag: Entity tag of the remote CI resource [required]
    """
    result = remoteci.delete(context, id=id, etag=etag)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Remote CI deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("remoteci-show", help="Show a remoteci.")
@click.option("--id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a Remote CI.

    >>> dcictl remoteci-show [OPTIONS]

    :param string id: ID of the remote CI to show [required]
    """
    result = remoteci.get(context, id=id)
    utils.format_output(result, context.format,
                        remoteci.RESOURCE[:-1], remoteci.TABLE_HEADERS)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = 'jobstates'
TABLE_HEADERS = ['id', 'status', 'comment', 'job_id', 'team_id',
                 'created_at']


def create(context, status, comment, job_id):
    jobstate = base.create(context, RESOURCE, status=status, comment=comment,
                           job_id=job_id)
    json = jobstate.json()
    context.last_jobstate_id = json['jobstate']['id']
    return jobstate


def list(context, where=None, embed=None):
    return base.list(context, RESOURCE, where=where, embed=embed)


def get(context, id, where=None, embed=None):
    return base.get(context, RESOURCE, id=id, where=where, embed=embed)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job
from dciclient.v1.api import jobdefinition

import json


def test_get_full_data(job_id, dci_context):
    full_data_job = job.get_full_data(dci_context, job_id)
    assert full_data_job['remoteci']['data'] == {'remoteci': 'remoteci'}
    assert full_data_job['jobdefinition']['name'] == 'tname'
    assert full_data_job['components'][0]['name'] == 'hihi'


def test_list(runner, dci_context, remoteci_id):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']

    result = runner.invoke(['team-list'])
    teams = json.loads(result.output)['teams']
    team_id = teams[0]['id']

    topic_team = runner.invoke(['topic-attach-team', '--id', topic['id'],
                                '--team_id', team_id])
    topic_team = json.loads(topic_team.output)

    result = runner.invoke(['test-create', '--name', 'foo', '--topic_id',
                            topic['id']])
    test = json.loads(result.output)['test']

    jd = runner.invoke(['jobdefinition-create', '--name', 'foo', '--test_id',
                       test['id'], '--topic_id', topic['id']])
    jd = json.loads(jd.output)['jobdefinition']

    component = runner.invoke(['component-create', '--name', 'foo',
                               '--type', 'foobar', '--topic_id', topic['id']])
    component = json.loads(component.output)['component']

    jobdefinition.add_component(dci_context, jd['id'], component['id'])

    job.schedule(dci_context, remoteci_id, topic['id'])
    l_job = runner.invoke(['job-list'])
    l_job = json.loads(l_job.output)
    assert len(l_job['jobs']) == 1
    assert l_job['jobs'][0]['remoteci_id'] == remoteci_id
    assert l_job['jobs'][0]['jobdefinition_id'] == jd['id']


def test_delete(runner, job_id):
    l_job = runner.invoke(['job-show', '--id', job_id])
    l_job_etag = json.loads(l_job.output)['job']['etag']

    result = runner.invoke(['job-delete', '--id', job_id,
                            '--etag', l_job_etag])
    result = json.loads(result.output)

    assert result['message'] == 'Job deleted.'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import context
from dciclient.v1.api import job
from dciclient.v1.api import jobstate
from dciclient.v1 import helper

from optparse import OptionParser

import sys


def get_pre_run_commands():

    commands = []

    return commands


def get_running_commands(component_details):

    cmpnt = component_details['components'][0]

    commands = [
        ['rm', '-rf', cmpnt['canonical_project_name']],
        ['git', 'clone', cmpnt['git']]
    ]

    commands.append({'cwd': '/var/tmp/%s' % cmpnt['canonical_project_name'],
                     'cmd': ['git', 'checkout', cmpnt['sha']]})
    commands.append({'cwd': '/var/tmp/%s' % cmpnt['canonical_project_name'],
                     'cmd': ['tox']})
    return commands


def retrieve_jobinformation(dci_context, remoteci_id, topic_id):
    """Retrieve job information"""
    scheduled_job = job.schedule(dci_context, remoteci_id, topic_id)
    if scheduled_job.status_code != 201:
        print("No job scheduled: '%s'" % scheduled_job.json())
        sys.exit(0)

    scheduled_job_id = scheduled_job.json()['job']['id']
    print("* Job scheduled: %s" % scheduled_job_id)
    full_data = job.get_full_data(dci_context, scheduled_job_id)

    return scheduled_job_id, full_data


def parse_command_line():
    parser = OptionParser("")
    parser.add_option("-u", "--dci-login", dest="dci_login",
                      help="DCI login")
    parser.add_option("-p", "--dci-password", dest="dci_password",
                      help="DCI password")
    parser.add_option("-a", "--dci-cs-url", dest="dci_cs_url",
                      help="DCI CS url")

    return parser.parse_args()


def main():
    (options, args) = parse_command_line()

    try:
        remoteci_id, team_id, topic_id = args
    except ValueError:
        print('dci-agent-tox remoteci_id team_id topic_id')
        sys.exit(1)

    dci_context = context.build_dci_context(options.dci_cs_url,
                                            options.dci_login,
                                            options.dci_password)

    new_job_id, jobinformation = retrieve_jobinformation(dci_context,
                                                         remoteci_id,
                                                         topic_id)

    # 1. Create the pre-run state
    pre_run_state = jobstate.create(dci_context,
                                    'pre-run',
                                    'Running pre-run commands',
                                    new_job_id) .json()
    pre_run_state_id = pre_run_state['jobstate']['id']

    # 2. Install requirements packages and tools.
    pre_run_commands = get_pre_run_commands()
    helper.run_commands(dci_context, pre_run_commands, '/var/tmp',
                        pre_run_state_id, new_job_id, team_id)

    # 5. Create the running state
    running_state = jobstate.create(dci_context, 'running',
                                    'Running commands',
                                    new_job_id).json()
    running_state_id = running_state['jobstate']['id']

    # 6. Run the echo helloworld command
    running_commands = get_running_commands(jobinformation)
    helper.run_commands(dci_context, running_commands, '/var/tmp',
                        running_state_id, new_job_id, team_id)

    # If we are there, then all the commands have succeed, create success state
    jobstate.create(dci_context, 'success', 'Tox commands run successfully',
                    new_job_id)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import context
from dciclient.v1 import helper

import click
import requests

CANDIDATES_URL = 'http://dci.enovance.com/candidates/'


def get_components(dci_context, url, topic_id):
    """Add the missing candidates in the components database"""
    rcs = []
    dci_candidate_components = []

    candidates = requests.get('%s/index.json' % url).json().keys()
    candidates.pop()  # Remove the 'latest' component that is a link

    registered_components = component.list(dci_context,
                                           topic_id).json()['components']

    for registered_component in registered_components:
        if registered_component['canonical_project_name'] == 'dci-candidate':
            rcs.append(registered_component['name'].replace(
                'dci-candidate ',
                '')
            )

    non_registered_candidates = list(set(candidates) - set(rcs))

    for candidate in non_registered_candidates:
        dci_candidate_components.append({
            'type': component.SNAPSHOT,
            'canonical_project_name': 'dci-candidate',
            'name': 'dci-candidate %s' % candidate,
            'url': '%s/%s' % (url, candidate),
            'topic_id': topic_id
        })

    return dci_candidate_components


@click.command()
@click.option('--dci-login', envvar='DCI_LOGIN', required=True,
              help="DCI username account.")
@click.option('--dci-password', envvar='DCI_PASSWORD', required=True,
              help="DCI password account.")
@click.option('--dci-cs-url', envvar='DCI_CS_URL', required=True,
              help="DCI CS url.")
@click.option('--dci-topic-id', envvar='DCI_TOPIC_ID', required=True,
              help="DCI topic id.")
def main(dci_login, dci_password, dci_cs_url, dci_topic_id):

    dci_context = context.build_dci_context(dci_cs_url, dci_login,
                                            dci_password)

    components = get_components(dci_context, CANDIDATES_URL, dci_topic_id)
    test_id = helper.get_test_id(dci_context, 'scenario01', dci_topic_id)

    helper.create_jobdefinition_and_add_component(dci_context, components,
                                                  test_id, dci_topic_id)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click
import requests
import yaml

import logging
import os
import sys
import traceback

from dciclient.v1.api import component
from dciclient.v1.api import context as dcicontext
from dciclient.v1.api import job as dcijob
from dciclient.v1.api import jobstate as dcijobstate
from dciclient.v1 import logger

from rdomhelper import logger as chainsaw_logger
from rdomhelper.provisioners.openstack import provisioner as os_provisioner


LOG = logging.getLogger('__chainsaw__')


def _check_url(url):
    resp = requests.get(url)
    if resp.status_code != 200:
        LOG.error("url '%s' is not available, content '%s'" %
                  (url, resp.content))
        raise


def _get_job_puddles(dci_context, job_id, mirror_repo_url):
    """Get the repositories of the puddles to test from the control server.
    Returns the content of the .repo files.
    """
    job_details = dcijob.get_full_data(dci_context, job_id)
    job_puddles = []
    for job_component in job_details['components']:
        if job_component['type'] == component.PUDDLE:
            puddle_name = job_component['canonical_project_name'].lower()
            repo_name = job_component['data']['repo_name']
            baseurl = "%s%s" % (mirror_repo_url, job_component['data']['path'])
            _check_url(baseurl)
            content = "[%s]\n"\
                      "name=%s\n"\
                      "baseurl=%s\n"\
                      "gpgcheck=0\n"\
                      "enabled=1\n" % (repo_name, repo_name, baseurl)

            job_puddles.append(
                {'type': 'yum_repo',
                 'content': content,
                 'dest': '/etc/yum.repos.d/%s.repo' % puddle_name})
    return job_puddles


@click.command()
@click.option('--os-auth-url', envvar='OS_AUTH_URL', required=True,
              help="Keystone auth url.")
@click.option('--os-username', envvar='OS_USERNAME', required=True,
              help="Openstack username account.")
@click.option('--os-password', envvar='OS_PASSWORD', required=True,
              help="Openstack password account.")
@click.option('--os-tenant-name', envvar='OS_TENANT_NAME', required=True,
              help="Openstack tenant name.")
@click.option('--dci-login', envvar='DCI_LOGIN', required=True,
              help="DCI username account.")
@click.option('--dci-password', envvar='DCI_PASSWORD', required=True,
              help="DCI password account.")
@click.option('--dci-cs-url', envvar='DCI_CS_URL', required=True,
              help="DCI CS url.")
@click.option('--dci-remoteci-id', envvar='DCI_REMOTECI_ID', required=True,
              help="DCI remoteci id.")
@click.option('--config-file', required=True, type=click.File('rb'),
              help="Chainsaw path configuration file.")
def cli(os_auth_url, os_username, os_password, os_tenant_name, dci_login,
        dci_password, dci_cs_url, dci_remoteci_id, config_file):
    config = yaml.load(config_file)
    mirror_repo_url = config['mirror_repo_url']

    dci_context = dcicontext.build_dci_context(
        dci_cs_url,
        dci_login,
        dci_password)
    dci_log_handler = logger.DciHandler(dci_context)
    chainsaw_logger.setup_logging(extra_handlers=(dci_log_handler,))
    status = 'pre-run'
    job = dcijob.schedule(dci_context, dci_remoteci_id)
    if job.status_code == 412:
        LOG.info("No jobs available for run.")
        sys.exit(0)
    elif job.status_code >= 400:
        LOG.error(job.text)
        sys.exit(1)
    job = job.json()
    job_id = job['job']['id']
    job_puddles = _get_job_puddles(dci_context, job_id, mirror_repo_url)

    try:
        rhsm_login = config['rhsm']['login']
        rhsm_password = config['rhsm'].get('password',
                                           os.environ.get('RHN_PW'))
        pool_id = config['rhsm'].get('pool_id')
        dcijobstate.create(dci_context, status, 'Creating the host0',
                           job_id)
        host0 = os_provisioner.deploy_host0(os_auth_url, os_username,
                                            os_password, os_tenant_name,
                                            config['provisioner'],
                                            config['ssh']['private_key'])

        provisioner_repositories = config['provisioner']['repositories']
        provisioner_repositories.extend(job_puddles)
        host0.configure(rhsm_login, rhsm_password, pool_id,
                        repositories=provisioner_repositories)

        dcijobstate.create(dci_context, status, 'Creating the undercloud',
                           job_id)
        vm_undercloud = host0.build_undercloud(
            config['undercloud']['guest_image_path'],
            config['undercloud']['guest_image_checksum'],
            rhsm_login=rhsm_login,
            rhsm_password=rhsm_password)

        status = 'running'
        dcijobstate.create(dci_context, status,
                           'Installing the undercloud machine', job_id)
        undercloud_repositories = config['undercloud']['repositories']
        undercloud_repositories.extend(job_puddles)
        vm_undercloud.configure(undercloud_repositories)
        vm_undercloud.install(config['undercloud']['guest_image_path'],
                              config['undercloud']['guest_image_checksum'])

        dcijobstate.create(dci_context, status, 'Deploying the overcloud',
                           job_id)
        vm_undercloud.deploy_overcloud(config['undercloud'].get('files', []))
        dcijobstate.create(dci_context, status, 'Running tempest', job_id)
        vm_undercloud.run_tempest()
        dcijobstate.create(dci_context, 'success', 'Job succeed :-)', job_id)
    except Exception as e:
        LOG.error(traceback.format_exc())
        dcijobstate.create(dci_context, 'failure', 'Job failed :-(', job_id)
        raise e

if __name__ == '__main__':
    cli()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import team
from dciclient.v1.api import topic
from dciclient.v1.helper import create_jobdefinition_and_add_component


def test_create_jobdefinition_and_add_component(runner, dci_context,
                                                components, test_id, topic_id):
    create_jobdefinition_and_add_component(dci_context, components, test_id,
                                           topic_id, jobdef_name='Test suite')

    team_id = team.list(dci_context).json()['teams'][0]['id']
    topic.attach_team(dci_context, topic_id, team_id)
    jdefs = jobdefinition.list(dci_context, topic_id).json()['jobdefinitions']
    jdef_names = [jdef['name'] for jdef in jdefs]

    jd_id = [jdef['id'] for jdef in jdefs if jdef['name'] == 'Test suite'][0]

    assert 'Test suite' in jdef_names

    jdef_cmpt = jobdefinition.get_components(dci_context, jd_id).json()
    jdef_cmpt = jdef_cmpt['components']

    assert 'component1' == jdef_cmpt[0]['name']
    assert 'component2' == jdef_cmpt[1]['name']



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = 'jobdefinitions'
TABLE_HEADERS = ['id', 'name', 'priority', 'test_id', 'etag', 'created_at',
                 'updated_at']


def create(context, name, topic_id, test_id=None, priority=None):
    return base.create(context, RESOURCE, name=name, test_id=test_id,
                       priority=priority, topic_id=topic_id)


def list(context, topic_id):
    return base.list(context, RESOURCE, topic_id=topic_id)


def get(context, id, where=None, embed=None):
    return base.get(context, RESOURCE, id=id, where=where, embed=embed)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def add_component(context, id, component_id):
    uri = '%s/%s/%s/components' % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={'component_id': component_id})


def get_components(context, id):
    uri = '%s/%s/%s/components' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def remove_component(context, id, component_id):
    uri = '%s/%s/%s/components/%s' % (context.dci_cs_api, RESOURCE, id,
                                      component_id)
    return context.session.delete(uri)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
error_message = """
This script make the use of Khaleesi mandatory which is not the current
situation. If you call it know, you will create jobdefinition that are not
compatible with the existing agents.
"""
raise Exception(error_message)
from dciclient.v1.api import component
from dciclient.v1.api import context
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import test

import requests

import time


def _get_timestamp(url):
    """Retrieve a timestamp from the last modified date of the resource."""
    result = requests.get(url)
    last_modified = result.headers['last-modified']
    return time.mktime(time.strptime(last_modified,
                                     '%a, %d %b %Y %H:%M:%S GMT'))


def get_khaleesi_component():
    """Retrieve Khaleesi component"""
    git_url = 'https://github.com/goneri/khaleesi'
    api_url = 'https://api.github.com/repos/goneri/khaleesi/branches/dci'
    commit = requests.get(api_url).json()['commit']

    message = commit['commit']['message']
    title = message.split('\n')[0]
    sha = commit['sha']
    ref = 'dci'
    url = commit['html_url']

    khaleesi_component = {
        "type": component.GIT_COMMIT,
        # leverage the sha to make a unique name
        "name": "[%s][%s]-%s" % ('khaleesi', 'master', sha),
        "title": title,
        "message": message,
        "sha": sha,
        "ref": ref,
        "url": url,
        "git": git_url,
        "canonical_project_name": 'khaleesi'
    }

    return khaleesi_component


def get_khaleesi_settings_component():
    """Retrieve Khaleesi Settings component"""
    git_url = 'file:///khaleesi-settings'

    khaleesi_settings_component = {
        "type": component.GIT_COMMIT,
        "name": "[%s][%s]" % ('khaleesi', 'master'),
        "git": git_url,
        "data": {},
        "canonical_project_name": 'khaleesi-settings'
    }

    return khaleesi_settings_component


def get_khaleesi_installer_component():

    kh_installer_component = {
        "type": component.KH_INSTALLER,
        # TODO(yassine): make a unique name
        "name": 'khaleesi-installer',
        "title": 'Khaleesi installer args',
        "data": {
            'kh_args': {
                'installer': 'rdo_manager',
                'installer-env': 'virthost',
                'installer-images': 'import',
                'installer-network': 'neutron',
                'installer-network-isolation': 'none',
                'installer-network-variant': 'ml2-vxlan',
                'installer-post_action': 'default',
                'installer-topology': 'minimal_no_ceph',
                'installer-tempest': 'smoke',
                'installer-updates': 'none'
            }
        }
    }

    return kh_installer_component


def get_khaleesi_puddle_component():
    url = 'http://download.lab.bos.redhat.com/rel-eng/OpenStack/7.0-RHEL-7'
    timestamp = _get_timestamp('%s/latest/status.txt' % url)

    khaleesi_puddle_component = {
        "type": component.PUDDLE,
        "name": 'Openstack puddle-%s' % timestamp,
        "title": 'Openstack puddle',
        "url": url,
        "data": {
            'kh_args': {
                'product': 'rhos',
                'product-repo': 'puddle',
                'product-version-build': 'latest',
                'product-version': '8_director',
                'extra-vars=product.build': 'latest',
                # TODO(yassine): get dynamically the versions
                'extra-vars=product.repo.puddle_pin_version': '2015-12-03.2',
                'extra-vars=product.repo.puddle_director_pin_version':
                    '2015-12-03.1'
            }
        }
    }

    return khaleesi_puddle_component


def get_test_id(dci_context, name):
    print("Use test '%s'" % name)
    test.create(dci_context, name)
    return test.get(dci_context, name).json()['test']['id']


if __name__ == '__main__':
    dci_context = context.build_dci_context()
    # Create Khaleesi-tempest test
    khaleesi_tempest_test_id = get_test_id(dci_context, 'Khaleesi-tempest')

    components = [get_khaleesi_component(),
                  get_khaleesi_settings_component(),
                  get_khaleesi_installer_component(),
                  get_khaleesi_puddle_component()]

    # If at least one component doesn't exist in the database then a new
    # jobdefinition must be created.
    at_least_one = False
    component_ids = []
    for cmpt in components:
        created_cmpt = component.create(dci_context, **cmpt)
        if created_cmpt.status_code == 201:
            print("Create component '%s', type '%s'" % (cmpt['name'],
                                                        cmpt['type']))
            component_ids.append(created_cmpt.json()['component']['id'])
            at_least_one = True

    if at_least_one:
        jobdef_name = "Khaleesi - OSP 8 - FV2"
        jobdef = jobdefinition.create(dci_context, jobdef_name,
                                      khaleesi_tempest_test_id)
        if jobdef.status_code == 201:
            jobdef_id = jobdef.json()['jobdefinition']['id']
            for cmpt_id in component_ids:
                jobdefinition.add_component(dci_context, jobdef_id, cmpt_id)
            print("Jobdefinition '%s' created." % jobdef_name)
        else:
            print("Error on jobdefinition creation: '%s'", jobdef.json())
    else:
        print("No jobdefinition created.")



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import file
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import jobstate
from dciclient.v1.api import test

import fcntl
import mimetypes
import os
import select
import subprocess

import six
import sys


def get_test_id(dci_context, name, topic_id):
    print("Use test '%s'" % name)
    test.create(dci_context, name, topic_id)
    return test.get(dci_context, name).json()['test']['id']


def create_jobdefinition_and_add_component(dci_context, components, test_id,
                                           topic_id, jobdef_name=None):
    # If at least one component doesn't exist in the database then a new
    # jobdefinition must be created.
    at_least_one = False
    component_ids = []
    names = []
    for cmpt in components:
        names.append(cmpt['name'])
        created_cmpt = component.create(dci_context, **cmpt)
        if created_cmpt.status_code == 201:
            at_least_one = True
        elif created_cmpt.status_code == 422:
            created_cmpt = component.get(dci_context, cmpt['name'])
        created_cmpt_name = created_cmpt.json()['component']['name']
        component_ids.append(created_cmpt.json()['component']['id'])

    if at_least_one:
        if jobdef_name is None:
            jobdef_name = created_cmpt_name
        jobdef = jobdefinition.create(dci_context, jobdef_name, topic_id,
                                      test_id)
        if jobdef.status_code == 201:
            jobdef_id = jobdef.json()['jobdefinition']['id']
            for cmpt_id in component_ids:
                jobdefinition.add_component(dci_context, jobdef_id, cmpt_id)
            print("Jobdefinition '%s' created." % jobdef_name)
        else:
            print("Error on jobdefinition creation: '%s'", jobdef.json())
    else:
        print("No jobdefinition created.")


def upload_file(context, path, job_id, mime=None):

    if not mime:
        mime = mimetypes.guess_type(path)[0]

    l_file = file.create(context, name=path, content=open(path, 'r').read(),
                         mime=mime, job_id=job_id)

    return l_file


def run_command(context, cmd, cwd, jobstate_id, team_id):
    """This function execute a command and send its log which will be
    attached to a jobstate.
    """
    output = six.StringIO()
    print('* Processing command: %s' % cmd)
    print('* Working directory: %s' % cwd)
    pipe_process = subprocess.Popen(cmd, cwd=cwd,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT)

    fcntl.fcntl(pipe_process.stdout.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)
    inputs = [pipe_process.stdout]
    outputs = []

    while True:
        readable, writable, exceptional = select.select(inputs, outputs,
                                                        inputs, 60)
        if not readable:
            break
        pstdout = pipe_process.stdout.read().decode('UTF-8', 'ignore')
        if len(pstdout) == 0:
            break
        else:
            print(pstdout)
            output.write(pstdout)

    pipe_process.wait()

    file.create(context, name='_'.join(cmd), content=output.getvalue(),
                mime='text/plain', jobstate_id=jobstate_id)
    output.close()
    return pipe_process.returncode


def run_commands(context, cmds, cwd, jobstate_id, job_id, team_id):
    for cmd in cmds:
        if isinstance(cmd, dict):
            rc = run_command(context, cmd['cmd'], cmd['cwd'], jobstate_id,
                             team_id)
        else:
            rc = run_command(context, cmd, cwd, jobstate_id, team_id)

        if rc != 0:
            error_msg = "Failed on command %s" % cmd
            jobstate.create(context, "failure", error_msg, job_id)
            print(error_msg)
            sys.exit(1)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import requests


class DCIConsumer(object):
    """A DCI consumer base class"""

    def __init__(self, dci_cs_url, login, password):
        self._s = self._get_http_session(dci_cs_url, login, password)

    def _get_http_session(self, dci_cs_url, login, password):
        session = requests.Session()
        session.headers.setdefault('Content-Type', 'application/json')
        session.auth = (login, password)
        session.dci_cs_url = dci_cs_url
        return session



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import codecs
import os
import setuptools

from dciclient import version


def _get_requirements():
    requirements_path = '%s/%s' % (os.path.dirname(os.path.abspath(__file__)),
                                   'requirements.txt')
    with open(requirements_path, 'r') as f:
        requirements = f.read()
        # remove the dependencies which comes from url source because
        # it's not supported by install_requires
        return [dep for dep in requirements.split('\n')
                if not dep.startswith('-e')]


def _get_readme():
    readme_path = '%s/%s' % (os.path.dirname(os.path.abspath(__file__)),
                             'README.md')

    with codecs.open(readme_path, 'r', encoding='utf8') as f:
        return f.read()


setuptools.setup(
    name='dciclient',
    version=version.__version__,
    packages=setuptools.find_packages(),
    author='Distributed CI team',
    author_email='distributed-ci@redhat.com',
    description='Python client for DCI Control Server',
    long_description=_get_readme(),
    install_requires=_get_requirements(),
    dependency_links=[
        'https://github.com/redhat-cip/python-rdo-m-helper.git'
        '#egg=python-rdo-m-helper'],
    url='https://github.com/redhat-cip/dci-control-server',
    license='Apache v2.0',
    include_package_data=True,
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Topic :: System :: Distributed Computing'
    ],
    entry_points={
        'console_scripts': [
            'dcictl = dciclient.shell:main'
        ],
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import context
from dciclient.v1.api import job
from dciclient.v1.api import jobstate
from dciclient.v1 import helper

from optparse import OptionParser

import platform
import sys

DCI_INFRA_URL = 'http://softwarefactory-project.io/r/dci-infra'


def _get_os_version():
    current_platform = platform.linux_distribution()

    if current_platform[0] == 'Fedora':
        current_os = 'f%s' % current_platform[1]
    elif current_platform[0] in ['CentOS Linux',
                                 'Red Hat Enterprise Linux Server']:
        current_os = 'el%s' % current_platform[1][:1]

    return current_os


def get_pre_run_commands(component_details, environment_url):

    current_os = _get_os_version()
    cmpnt = component_details['components'][0]

    commands = [
        ['rm', '-rf', '/var/tmp/dci-infra'],
        ['sudo', 'rm', '/etc/yum.repos.d/dci.repo'],
        ['sudo', 'curl', '-o', '/etc/yum.repos.d/dci.repo', '%s/dci%s.repo' %
            (cmpnt['url'],
             current_os)],
        ['git', 'clone', DCI_INFRA_URL],
    ]

    commands.append({'cwd': '/var/tmp/dci-infra',
                     'cmd': ['git', 'clone', environment_url, 'data']})

    return commands


def get_running_commands():

    commands = []

    commands.append({'cwd': '/var/tmp/dci-infra',
                     'cmd': ['ansible-galaxy', 'install', '-r',
                             'installed_roles', '-p', 'roles/']})
    commands.append({'cwd': '/var/tmp/dci-infra',
                     'cmd': ['ansible-playbook', '-i', 'data/hosts',
                             'playbook.yml', '--tags', 'dci-core']})

    return commands


def get_post_run_commands():

    commands = []

    commands.append({'cwd': '/var/tmp/dci-infra',
                     'cmd': ['rake', 'serverspec:functional']})

    return commands


def retrieve_jobinformation(dci_context, remoteci_id, topic_id):
    """Retrieve job information"""
    scheduled_job = job.schedule(dci_context, remoteci_id, topic_id)
    if scheduled_job.status_code != 201:
        print("No job scheduled: '%s'" % scheduled_job.json())
        sys.exit(0)

    scheduled_job_id = scheduled_job.json()['job']['id']
    print("* Job scheduled: %s" % scheduled_job_id)
    full_data = job.get_full_data(dci_context, scheduled_job_id)

    return scheduled_job_id, full_data


def parse_command_line():
    parser = OptionParser("")
    parser.add_option("-u", "--dci-login", dest="dci_login",
                      help="DCI login")
    parser.add_option("-p", "--dci-password", dest="dci_password",
                      help="DCI password")
    parser.add_option("-a", "--dci-cs-url", dest="dci_cs_url",
                      help="DCI CS url")

    return parser.parse_args()


def main():
    (options, args) = parse_command_line()

    try:
        remoteci_id, team_id, topic_id, environment_url = args
    except ValueError:
        print('dci-agent-dci remoteci_id team_id topic_id environment_url')
        sys.exit(1)

    dci_context = context.build_dci_context(options.dci_cs_url,
                                            options.dci_login,
                                            options.dci_password)

    new_job_id, jobinformation = retrieve_jobinformation(dci_context,
                                                         remoteci_id,
                                                         topic_id)

    # 1. Create the pre-run state
    pre_run_state = jobstate.create(dci_context,
                                    'pre-run',
                                    'Running pre-run commands',
                                    new_job_id) .json()
    pre_run_state_id = pre_run_state['jobstate']['id']

    # 2. Install requirements packages and tools.
    pre_run_commands = get_pre_run_commands(jobinformation, environment_url)
    helper.run_commands(dci_context, pre_run_commands, '/var/tmp',
                        pre_run_state_id, new_job_id, team_id)

    # 3. Create the running state
    running_state = jobstate.create(dci_context, 'running',
                                    'Running commands',
                                    new_job_id).json()
    running_state_id = running_state['jobstate']['id']

    # 4. Run the ansible deployment
    running_commands = get_running_commands()
    helper.run_commands(dci_context, running_commands, '/var/tmp',
                        running_state_id, new_job_id, team_id)

    # 5. Create the post-run state
    post_run_state = jobstate.create(dci_context,
                                     'post-run',
                                     'Running post-run commands',
                                     new_job_id) .json()
    post_run_state_id = post_run_state['jobstate']['id']

    # 6. Run the test suite
    post_run_commands = get_post_run_commands()
    helper.run_commands(dci_context, post_run_commands, '/var/tmp',
                        post_run_state_id, new_job_id, team_id)

    # If we are there, then all the commands have succeed, create success state
    jobstate.create(dci_context, 'success', 'Rake commands run successfully',
                    new_job_id)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.consumers import dciconsumer
from dciclient.v1.handlers import file
from dciclient.v1.handlers import job
from dciclient.v1.handlers import jobdefinition
from dciclient.v1.handlers import jobstate
from dciclient.v1.handlers import remoteci

import subprocess


class Agent(dciconsumer.DCIConsumer):
    """A DCI agent class"""

    def __init__(self, dci_cs_url, login, password, remoteci_name):
        super(Agent, self).__init__(dci_cs_url, login, password)
        self.remoteci_id, self.team_id = self._retrieve_context(remoteci_name)
        self.job_id, self.jobinformation = self._retrieve_jobinformation()

    def _retrieve_context(self, remoteci_name):
        """Retrieve a remoteci id and team_id by the remote_ci name"""
        l_remoteci = remoteci.RemoteCI(self._s)

        remoteci_data = l_remoteci.get(remoteci_name).json()['remoteci']

        return remoteci_data['id'], remoteci_data['team_id']

    def _retrieve_jobinformation(self):
        """Retrieve job informations"""
        job_embed = ['jobdefinition']

        l_job = job.Job(self._s)
        l_jobdefinition = jobdefinition.JobDefinition(self._s)

        job_data = l_job.schedule(remoteci_id=self.remoteci_id).json()['job']
        jobdefinition_data = (
            l_job .get(id=job_data['id'], embed=','.join(job_embed))
            .json()['job']['jobdefinition']
        )
        component_data = (
            l_jobdefinition.get_components(jobdefinition_data['id'])
            .json()['components']
        )

        jobinformation = {
            'job': job_data,
            'jobdefinition': jobdefinition_data,
            'components': component_data,
            'data': l_job.get_full_data(job_data['id']),
        }

        return job_data['id'], jobinformation

    def _get_state_hash(self, status, comment):
        return {'job_id': self.job_id,
                'status': status,
                'comment': comment,
                'team_id': self.team_id}

    def _run_command(self, cmd, cwd, jobstate_id):
        """Run the actual command and upload the output """
        l_file = file.File(self._s)
        ret_value = 0

        cwd = cmd[0]['cwd'] if 'cwd' in cmd[0] else cwd
        if 'cwd' in cmd[0]:
            cmd = cmd[1:]

        try:
            flat_cmd = ' '.join(cmd)
            output = subprocess.check_output(flat_cmd, shell=True, cwd=cwd)
            l_file.create(name='_'.join(cmd),
                          content=output,
                          mime='text/plain',
                          jobstate_id=jobstate_id,
                          team_id=self.team_id)
        except subprocess.CalledProcessError as e:
            l_file.create(name=e.cmd,
                          content=e.output,
                          mime='text/plain',
                          jobstate_id=jobstate_id,
                          team_id=self.team_id)
            ret_value = -1
        except (OSError, IOError, TypeError):
            ret_value = -1

        return ret_value

    def run_agent(self, run, pre_run=None, post_run=None):
        """Run the agent business logic"""
        failure = False
        l_jobstate = jobstate.JobState(self._s)
        kwargs = self._get_state_hash('new', 'Starting run agent')
        l_jobstate.create(**kwargs)

        if pre_run:
            kwargs = (
                self
                ._get_state_hash('pre-run', 'Starting pre_run setup')
            )
            pre_run_jobstate_id = (
                l_jobstate
                .create(**kwargs).json()['jobstate']['id']
            )
            for cmd in pre_run['cmds']:
                rc = (
                    self
                    ._run_command(cmd, pre_run['cwd'], pre_run_jobstate_id)
                )
                if rc != 0:
                    kwargs = (
                        self
                        ._get_state_hash('failure', 'Failure in pre_run setup')
                    )
                    l_jobstate.create(**kwargs)
                    failure = True

        if not failure:
            kwargs = self._get_state_hash('running', 'Starting run setup')
            run_jobstate_id = (
                l_jobstate.create(**kwargs)
                .json()['jobstate']['id']
            )
            for cmd in run['cmds']:
                rc = self._run_command(cmd, run['cwd'], run_jobstate_id)
                if rc != 0:
                    kwargs = (
                        self
                        ._get_state_hash('failure', 'Failure in run setup')
                    )
                    l_jobstate.create(**kwargs)
                    failure = True

        if not failure and post_run:
            kwargs = (self
                      ._get_state_hash('post-run', 'Starting post_run setup'))
            post_run_jobstate_id = (
                l_jobstate
                .create(**kwargs).json()['jobstate']['id']
            )
            for cmd in post_run['cmds']:
                self._run_command(cmd, post_run['cwd'], post_run_jobstate_id)
                if rc != 0:
                    kwargs = (
                        self
                        ._get_state_hash('failure', 'Fail in post_run setup')
                    )
                    l_jobstate.create(**kwargs)
                    failure = True

        if not failure:
            kwargs = self._get_state_hash('success', 'Finished running agent')
            l_jobstate.create(**kwargs)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base

import json

RESOURCE = 'components'
TABLE_HEADERS = ['id', 'name', 'canonical_project_name',
                 'type', 'sha', 'title', 'message', 'url',
                 'git', 'ref', 'data', 'created_at']

GIT_COMMIT = 'git_commit'
KH_INSTALLER = 'kh_installer'
PUDDLE = 'puddle'
SNAPSHOT = 'snapshot'


def create(context, name, type, topic_id, canonical_project_name=None, data={},
           sha=None, title=None, message=None, url=None, git=None, ref=None):
    return base.create(context, RESOURCE, name=name, type=type,
                       canonical_project_name=canonical_project_name,
                       data=json.dumps(data), sha=sha, title=title,
                       message=message, url=url, git=git, ref=ref,
                       topic_id=topic_id)


def list(context, topic_id):
    return base.list(context, RESOURCE, topic_id=topic_id)


def get(context, id, where=None, embed=None):
    return base.get(context, RESOURCE, id=id, where=where, embed=embed)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
import json


def test_prettytable_output(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    result = runner.invoke(['--format', 'table', 'team-show', '--id',
                            team['id']])

    output = result.output.split('\n')
    # NOTE(spredzy) : The expected output for a table format looks like the
    #                 following :
    #
    # +------------
    # |  id | name
    # +------------
    # | id1 | name1
    # | id2 | name2
    # | id3 | name3
    # +------------
    #
    # The header variable below represents the header data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    #
    # The data variable below represents the actual data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    header = ' '.join(output[1].split())
    data = ' '.join(output[3].split())

    expected_data = (team['id'], team['name'], team['etag'],
                     team['created_at'], team['updated_at'])

    assert header == '| id | name | etag | created_at | updated_at |'

    assert data == '| %s | %s | %s | %s | %s |' % expected_data


def test_list(runner):
    runner.invoke(['team-create', '--name', 'foo'])
    runner.invoke(['team-create', '--name', 'bar'])
    result = runner.invoke(['team-list'])
    teams = json.loads(result.output)['teams']
    # NOTE (spredzy): We put 4 because of the 2 creates plus
    # admin and user provisionned during server test
    assert len(teams) == 4


def test_create(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']
    assert team['name'] == 'foo'


def test_update(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    result = runner.invoke(['team-update', '--id', team['id'],
                            '--etag', team['etag'], '--name', 'bar'])
    result = json.loads(result.output)

    assert result['message'] == 'Team updated.'
    assert result['id'] == team['id']


def test_delete(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    result = runner.invoke(['team-delete', '--id', team['id'],
                            '--etag', team['etag']])
    result = json.loads(result.output)

    assert result['message'] == 'Team deleted.'


def test_show(runner):
    result = runner.invoke(['team-create', '--name', 'foo'])
    team = json.loads(result.output)['team']

    result = runner.invoke(['team-show', '--id', team['id']])
    team = json.loads(result.output)['team']

    assert team['name'] == 'foo'



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.api import context as dci_context


_default_dci_cs_url = 'http://127.0.0.1:5000'


from click.core import Command
from click.decorators import _make_command


# Override Click command so that to preserve the function's docstring,
# this is useful for sphinx autodoc generation
def command(name=None, cls=None, **attrs):
    if cls is None:
        cls = Command

    def decorator(f):
        r = _make_command(f, name, attrs, cls)
        r.__doc__ = f.__doc__
        return r
    return decorator

click.core.command = command


@click.group()
@click.option('--dci-login', envvar='DCI_LOGIN', required=True,
              help="DCI login or 'DCI_LOGIN' environment variable.")
@click.option('--dci-password', envvar='DCI_PASSWORD', required=True,
              help="DCI password or 'DCI_PASSWORD' environment variable.")
@click.option('--dci-cs-url', envvar='DCI_CS_URL', default=_default_dci_cs_url,
              help="DCI control server url, default to '%s'." %
                   _default_dci_cs_url)
@click.option('--format', envvar='DCI_CLI_OUTPUT_FORMAT', required=False,
              default='table', help="DCI CLI output format.")
@click.pass_context
def cli(ctx, dci_login, dci_password, dci_cs_url, format):
    context = dci_context.build_dci_context(dci_login=dci_login,
                                            dci_password=dci_password,
                                            dci_cs_url=dci_cs_url)
    context.format = format
    ctx.obj = context

import dciclient.v1.shell_commands.component  # noqa
import dciclient.v1.shell_commands.file  # noqa
import dciclient.v1.shell_commands.job  # noqa
import dciclient.v1.shell_commands.jobdefinition  # noqa
import dciclient.v1.shell_commands.jobstate  # noqa
import dciclient.v1.shell_commands.remoteci  # noqa
import dciclient.v1.shell_commands.team  # noqa
import dciclient.v1.shell_commands.test  # noqa
import dciclient.v1.shell_commands.topic  # noqa
import dciclient.v1.shell_commands.user  # noqa



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.consumers import dciconsumer
from dciclient.v1.handlers import component
from dciclient.v1.handlers import jobdefinition
from dciclient.v1.handlers import test


class Feeder(dciconsumer.DCIConsumer):
    """A DCI feeder class"""

    def __init__(self, dci_cs_url, login, password):
        super(Feeder, self).__init__(dci_cs_url, login, password)
        self.component_ids = []

    def ensure_test(self, test_name):
        """Ensure test exist"""
        l_test = test.Test(self._s)
        return l_test.create(name=test_name).json()['test']['id']

    def create_component(self, component_data):
        """Create an entry in Component"""
        l_component = component.Component(self._s)

        result = l_component.create(**component_data).json()
        self.component_ids.append(result['component']['id'])

    def create_jobdefinition(self, name, test):
        """Create an entry in JobDefinition"""
        l_jobdefinition = jobdefinition.JobDefinition(self._s)

        jobdefinition_id = l_jobdefinition.create(
            name=name,
            test_id=test,
        ).json()['jobdefinition']['id']

        for component_id in self.component_ids:
            l_jobdefinition.add_component(
                id=jobdefinition_id,
                component_id=component_id,
            )



# -*- encoding: utf-8 -*-
#
# Copyright 2015 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals
import json


def test_prettytable_output(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']
    assert topic['name'] == 'osp'

    result = runner.invoke(['component-create', '--name', 'foo',
                            '--type', 'bar', '--topic_id', topic['id']])
    component = json.loads(result.output)['component']

    result = runner.invoke(['--format', 'table', 'component-show', '--id',
                            component['id']])

    output = result.output.split('\n')
    # NOTE(spredzy) : The expected output for a table format looks like the
    #                 following :
    #
    # +------------
    # |  id | name
    # +------------
    # | id1 | name1
    # | id2 | name2
    # | id3 | name3
    # +------------
    #
    # The header variable below represents the header data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    #
    # The data variable below represents the actual data, when more than one
    # space is located between the string and the '|' space number is shrink to
    # one ( this is what ' '.join(string.split()) does
    header = ' '.join(output[1].split())
    data = ' '.join(output[3].split())

    expected_data = (component['id'], component['name'],
                     component['canonical_project_name'],
                     component['type'], component['sha'],
                     component['title'], component['message'],
                     component['url'], component['git'], component['ref'],
                     component['created_at'])

    assert header == ('| id | name | canonical_project_name '
                      '| type | sha | title | message | url | git '
                      '| ref | data | created_at |')

    assert data == ('| %s | %s | %s | %s | %s | %s | %s | %s | %s | %s | {} '
                    '| %s |' % expected_data)


def test_list(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']

    result = runner.invoke(['team-list'])
    teams = json.loads(result.output)['teams']
    team_id = teams[0]['id']

    topic_team = runner.invoke(['topic-attach-team', '--id', topic['id'],
                                '--team_id', team_id])
    topic_team = json.loads(topic_team.output)

    runner.invoke(['component-create', '--name', 'foo', '--type', 'bar',
                   '--topic_id', topic['id']])
    runner.invoke(['component-create', '--name', 'bar', '--type', 'bar2',
                   '--topic_id', topic['id']])
    result = runner.invoke(['component-list', '--topic_id', topic['id']])
    components = json.loads(result.output)['components']

    assert len(components) == 2
    assert components[0]['name'] == 'foo'
    assert components[1]['name'] == 'bar'


def test_create(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']

    component = runner.invoke(['component-create', '--name', 'foo',
                               '--type', 'foobar', '--topic_id', topic['id']])
    component = json.loads(component.output)['component']
    assert component['name'] == 'foo'


def test_delete(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']

    result = runner.invoke(['component-create', '--name', 'foo',
                            '--type', 'bar', '--topic_id', topic['id']])
    component = json.loads(result.output)['component']

    result = runner.invoke(['component-delete', '--id', component['id']])
    result = json.loads(result.output)

    assert result['message'] == 'Component deleted.'


def test_show(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])
    topic = json.loads(topic.output)['topic']
    assert topic['name'] == 'osp'

    result = runner.invoke(['component-create', '--name', 'foo',
                            '--type', 'bar', '--topic_id', topic['id']])
    component = json.loads(result.output)['component']

    result = runner.invoke(['component-show', '--id', component['id']])
    component = json.loads(result.output)['component']

    assert component['name'] == 'foo'
